;****************************************************************************************************
;* 		  TWI- Master Routinen																		*
;*      ------------------------																	*
;*		1)	TWI_START_CONDITION					// Lst eine Startbedienung aus.					*
;*		2)	TWI_STOP_CONDITION					// Lst eine Stoppbedienung aus.					*
;*		3)	TWI_SLA_W							// Sendet SLA+W										*
;*		4)	TWI_SLA_R							// Sendet SLA+R										*
;*		5)	WAIT_TWINT							// Wartet bis das TWINT- Flag gesetzt wurde			*
;*		6)	TWI_MT_Data							// bertrgt Daten zum Slave						*
;*		7)	TWI_MR_Data							// Empfngt Daten vom Slave und sendet ACK			*
;*		8)	TWI_MR_Data_NACK					// Empfngt Daten vom Slave und sendet NACK			*
;*		9)  TWI_MASTER_INIT						// Iniitalisert TWI als Master						*
;*						   																		    *
;*		Used GPReg:																					*
;*     ---------------																				*
;*	temp		=	r16																				*
;*	temp2		=	r17																				*
;****************************************************************************************************
;		.equ	TWBR_val		= 64
;		.equ	TWI_Prescaler	= 1				// 1= Clock divided by 4







;*   Status Codes for Master Transmitter Mode
;* --------------------------------------------
 		.equ	MT_Start_Condition   			= $08			// A START condition has been transmitted
		.equ	MT_Repeated_Start				= $10			// A repeated START condition has been transmitted
		.equ	MT_SLA_W_ACK					= $18			// SLA+W has been transmitted; ACK has been received
		.equ	MT_SLA_W_NACK					= $20			// SLA+W has been transmitted; NOT ACK has been received
		.equ	MT_DATA_ACK						= $28			// Data byte has been transmitted; ACK has been received
		.equ	MT_DATA_NACK					= $30			// Data byte has been transmitted; NOT ACK has been received
		.equ	MT_Arbitration_lost				= $38			// Arbitration lost in SLA+W or data bytes


;*   Status Codes for Master Receiver Mode
;* -----------------------------------------
		.equ	MR_Start_Condition				= $08			// A START condition has been transmitted
		.equ	MR_Repeated_Start				= $10			// A repeated START condition has been transmitted
		.equ	MR_Arbitration_lost				= $38			// Arbitration lost in SLA+R or NOT	ACK bit
		.equ	MR_SLA_R_ACK					= $40			// SLA+R has been transmitted; ACK has been received
		.equ	MR_SLA_R__NACK					= $48			// SLA+R has been transmitted; NOT ACK has been received
		.equ	MR_DATA_ACK						= $50			// Data byte has been received; ACK has been returned
		.equ	MR_DATA_NACK					= $58			// Data byte has been received; NOT ACK has been returned





;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
TWI_START_CONDITION:
		ldi		temp, (1<<TWINT)|(0<<TWEA)|(1<<TWSTA)|(0<<TWSTO)|(0<<TWWC)|(1<<TWEN)|(0<<1)|(0<<TWIE)
		OUT		TWCR, temp
		rcall	WAIT_TWINT
	ret
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
TWI_STOP_CONDITION:
		ldi		temp, (1<<TWINT)|(0<<TWEA)|(0<<TWSTA)|(1<<TWSTO)|(0<<TWWC)|(1<<TWEN)|(0<<1)|(0<<TWIE)
		OUT		TWCR, temp
	ret

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
TWI_SLA_W:										// Input: temp = Slave Address
		andi	temp, 0b1111111_0				// TWI Write =0(Bit0) 
		OUT		TWDR, temp
		ldi		temp, (1<<TWINT)|(0<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|(0<<TWWC)|(1<<TWEN)|(0<<1)|(0<<TWIE)
		OUT		TWCR, temp
		rcall	WAIT_TWINT
	ret

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
TWI_SLA_R:										// Input: temp = Slave Address
		ori		temp, 0b0000000_1				// TWI Write =0(Bit0) 
		OUT		TWDR, temp
		ldi		temp, (1<<TWINT)|(0<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|(0<<TWWC)|(1<<TWEN)|(0<<1)|(0<<TWIE)
		OUT		TWCR, temp
		rcall	WAIT_TWINT
	ret									
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
TWI_MT_Data:
		OUT		TWDR, temp
		ldi		temp, (1<<TWINT)|(0<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|(0<<TWWC)|(1<<TWEN)|(0<<1)|(0<<TWIE)
		OUT		TWCR, temp
		rcall	WAIT_TWINT
	ret	
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
TWI_MR_Data:
		ldi		temp, (1<<TWINT)|(1<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|(0<<TWWC)|(1<<TWEN)|(0<<1)|(0<<TWIE)
		OUT		TWCR, temp
		rcall	WAIT_TWINT						// Warte bis TWINT gesetzt wird
		IN		temp2, TWDR						// 
	ret	
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
TWI_MR_Data_NACK:
		ldi		temp, (1<<TWINT)|(0<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|(0<<TWWC)|(1<<TWEN)|(0<<1)|(0<<TWIE)
		OUT		TWCR, temp
		rcall	WAIT_TWINT						// Warte bis TWINT gesetzt wird
		IN		temp2, TWDR							// 
	ret	
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
; Parameter: none
; Return: temp= TWSR
WAIT_TWINT:										
 		IN		temp, TWCR						// Statusregister Kopieren
		sbrs	temp, TWINT						// berspringe eine Zeile falls TWINT- Flag gesetzt wurde
		rjmp	WAIT_TWINT						// Warte bis TWINT gesetzt wird
		IN		temp, TWSR						// TWI Status Register auslesen
		andi	temp, $F8						// Prescaler Bits(TWPSx) ausmaskieren 
	ret	
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
; Parameter: const char unsigned TWBR_val = x		// Baudrate
; Parameter: const char unsigned TWI_Prescaler =x	// Prescaler Settings
; Return: none
TWI_MASTER_INIT:
; TWI TWBR 											Bit RATE Register	
		ldi 	temp2, TWBR_val
		OUT 	TWBR, temp2
; TWI TWSR 											STATUS-Register		
		ldi 	temp2, TWI_Prescaler
		OUT 	TWSR, temp2
; TWI TWCR 											Control-REGISTER	
		ldi 	temp2, 0<<TWINT|1<<TWEA|1<<TWEN
		OUT 	TWCR, temp2
	ret
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
; 	Parameter: None 
; 	Return: temp= Error State
TWI_MASTER_ERROR:										
		IN		temp, TWSR								// ERROR State- Aufnahme
		andi	temp, 0xF8								// Prescaler Bits ausmaskieren	
		ldi 	temp2, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)	// TWI STOP
		OUT 	TWCR, temp2								// 
		ldi 	temp2, (0<<TWEN)						// TWI Disable
		OUT 	TWCR, temp2								//
		rcall	TWI_MASTER_INIT							// Neu Initialisieren
	ret
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX	






